/**
 * @file	cf_log.h
 * @author	myusgun <myusgun@gmail.com>
 * @version	0.1
 *
 * @section 샘플코드
 * @code

#include "cf_log.h"

CF_Log_Ctx gLogCtx;

void main (const char * message)
{
	int			i, j;
	char		logname[16] = {0x00,};

	CF_DEBUG_CALLSTACK_PUSH (gDebugCtx);

	CF_Log_Initialize (10);

	// with context
	gLogCtx = CF_Log_CreateCtx (file, CF_LOG_BUFFER_DEFAULT);
	if (gLogCtx == NULL)
		CF_DEBUG_PRINT (stderr, "create log ctx error\n");

	for (i = 0 ; i < 10000 ; i++)
		CF_Log_Write (gLogCtx, "LOG_TEST", "turn %d\n", i);

	CF_DEBUG_CALLSTACK_POP (gDebugCtx, &gDebugCallstack);

	CF_Log_Write (gLogCtx, message,
							"here is the end of function [file:%s line:%d func:%s]\n",
							gDebugCallstack.file,
							gDebugCallstack.line,
							gDebugCallstack.function);

	CF_Log_DestroyCtx (gLogCtx);

	// with id number
	for (i = 0 ; i < 10 ; i++)
	{
		sprintf (logname, "logid%d.txt", i);
		CF_LOG_OPEN (i, logname, CF_LOG_BUFFER_NO);
	}

	for (i = 0 ; i < 10 ; i++)
	{
		for (j = 0 ; j < 10000 ; j++)
			CF_LOG_WRITE (i, "LOG_ID_TEST", "turn %d\n", j);

		CF_LOG_CLOSE (i);
	}

	CF_Log_Finalize ();
}

 * @endcode
 */
#ifndef __CF_LOG_H__
#define __CF_LOG_H__

#include "cf_base.h"

#define CF_ERROR_LOG_INITIALIZE			CF_ERROR_LOG - 1
#define CF_ERROR_LOG_INVALID_CTX		CF_ERROR_LOG - 2
#define CF_ERROR_LOG_SET_MULTITHREAD	CF_ERROR_LOG - 3
#define CF_ERROR_LOG_UNSET_MULTITHREAD	CF_ERROR_LOG - 4
#define CF_ERROR_LOG_LOCK_CTX			CF_ERROR_LOG - 5
#define CF_ERROR_LOG_UNLOCK_CTX			CF_ERROR_LOG - 6
#define CF_ERROR_LOG_FLUSH				CF_ERROR_LOG - 7
#define CF_ERROR_LOG_INVALID_MAPID		CF_ERROR_LOG - 8
#define CF_ERROR_LOG_NOT_MAPPED_ID		CF_ERROR_LOG - 9
#define CF_ERROR_LOG_ALREADY_MAPPED_ID	CF_ERROR_LOG - 10
#define CF_ERROR_LOG_NOT_INITIALIZE		CF_ERROR_LOG - 11

/**
 * 로그의 버퍼 크기를 기본 값으로 설정
 *
 * @see CF_Log_CreateCtx, CF_LOG_OPEN
 */
#define CF_LOG_BUFFER_DEFAULT			-1

/** 로그 기록 시 버퍼를 사용하지 않고 즉시 기록
 *
 * @see CF_Log_CreateCtx, CF_LOG_OPEN, CF_Log_Initialize
 */
#define CF_LOG_BUFFER_NO				0

/**
 * 로그에 아이디 넘버를 부여하여 생성 <br />
 * 로그 기록 시, 아이디 넘버를 사용하면 해당 로그로 기록할 수 있음
 *
 * @param __id		부여할 아이디 넘버
 * @param __file	로그 파일 경로
 * @param __memsize	로그 버퍼 크기
 *
 * @see CF_Log_MapCtxID
 */
#define CF_LOG_OPEN(__id,__file,__memsize)				\
	CF_Log_MapCtxID (__id, CF_Log_CreateCtx (__file, __memsize))

/**
 * 아이디 넘버에 해당하는 로그에 쓰기
 *
 * @param __id	로그의 아이디 넘버
 * @param __pf	로그의 프리픽스 문자열
 * @param __fmt	포맷 스트링
 * @param ...	가변 인자
 *
 * @see CF_Log_GetMappedCtx
 */
#define CF_LOG_WRITE(__id,__pf,__fmt,...)				\
	CF_Log_Write (CF_Log_GetMappedCtx (__id),__pf,__fmt,##__VA_ARGS__)

/**
 * 아이디 넘버에 해당하는 로그를 닫고 해제
 *
 * @param __id 로그의 아이디 넘버
 *
 * @see CF_Log_UnmapCtxID
 */
#define CF_LOG_CLOSE(__id)		CF_Log_UnmapCtxID (__id)

/** 로그 컨텍스트 */
typedef void *	CF_Log_Ctx;

#ifdef __cplusplus
extern "C" {
#endif

/**
 * 로그를 사용하기 위해 초기화
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param logPool 아이디 넘버 최대 값
 */
CF_EXPORT int
CF_Log_Initialize		(const int logPool);

/**
 * 로그가 모두 사용된 후 자원 해제
 *
 * @return CF_OK 반환
 */
CF_EXPORT int
CF_Log_Finalize			(void);

/**
 * 로그 컨텍스트 생성
 *
 * @return 성공 시, 로그 컨텍스트; 실패 시, NULL
 *
 * @param path		로그 파일 경로
 * @param memsize	로그 버퍼 크기
 *
 * @see CF_LOG_BUFFER_DEFAULT, CF_LOG_BUFFER_NO
 */
CF_EXPORT CF_Log_Ctx
CF_Log_CreateCtx		(const char	* path,
						 const int	memsize);

/**
 * 로그 컨텍스트 해제
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드 
 *
 * @param ctx 로그 컨텍스트
 */
CF_EXPORT int
CF_Log_DestroyCtx		(CF_Log_Ctx ctx);

/**
 * 로그 컨텍스트에 멀티쓰레드 모드 설정
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드 
 *
 * @param ctx 로그 컨텍스트
 */
CF_EXPORT int
CF_Log_SetMultiThread	(CF_Log_Ctx ctx);

/**
 * 로그 컨텍스트에 멀티쓰레드 모드 설정 해제
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드 
 *
 * @param ctx 로그 컨텍스트
 */
CF_EXPORT int
CF_Log_UnsetMultiThread	(CF_Log_Ctx ctx);

/**
 * 로그 컨텍스트에 따라 로그 쓰기
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param ctx		로그 컨텍스트
 * @param prefix	로그의 프리픽스 문자열
 * @param fmt		포맷 스트링
 * @param ...		가변 인자
 */
CF_EXPORT int
CF_Log_Write			(CF_Log_Ctx	ctx,
						 const char	* prefix,
						 const char	* fmt, ...);

/**
 * 로그 버퍼의 데이터를 즉시 로그 파일에 쓰기
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param ctx 로그 컨텍스트
 */
CF_EXPORT int
CF_Log_Flush			(CF_Log_Ctx ctx);

/**
 * 로그 컨텍스트에 아이디 넘버 할당<br />
 * 로그 기록 시, 아이디 넘버를 사용하면 해당 로그로 기록할 수 있음
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param mapid	부여할 아이디 넘버
 * @param ctx	로그 컨텍스트
 *
 * @remark 반드시 먼저 초기화 해야하며, 초기화 시에 주어진 번호보다 작은 아이디 넘버를 사용해야 함
 *
 * @see CF_LOG_OPEN, CF_Log_CreateCtx
 */
CF_EXPORT int
CF_Log_MapCtxID			(const int			mapid,
						 const CF_Log_Ctx	ctx);

/**
 * 아이디 넘버에 해당하는 로그를 닫고 해당하는 컨텍스트를 해제
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param mapid 로그의 아이디 넘버
 *
 * @remark 아이디 넘버에 해당하는 컨텍스트가 해제되므로 주의
 *
 * @see CF_LOG_CLOSE, CF_Log_DestroyCtx
 */
CF_EXPORT int
CF_Log_UnmapCtxID		(const int mapid);

/**
 * 아이디 넘버에 해당하는 로그 컨텍스트를 얻기
 *
 * @return 성공 시, 로그 컨텍스트; 실패 시, NULL
 *
 * @param mapid 로그의 아이디 넘버
 */
CF_EXPORT CF_Log_Ctx
CF_Log_GetMappedCtx		(const int mapid);

#ifdef __cplusplus
}
#endif

#endif // #ifndef __CF_LOG_H__
